import express from 'express';
import cors from 'cors';
import path from 'path';
import process from 'process';
import moment from 'moment';
import cookieParser from 'cookie-parser';
import session from 'express-session';

import connectDB from './db.js';
import userService from './data/userService.js';
import articleService from './data/articleService.js';

const app = express();
const PORT = 8080;

const sessionTime = 5; // 세션시간(임시)

connectDB();

app.use(express.static(path.join(process.cwd(), '../frontend/build')));

const maxAge = 1000 * 60 * sessionTime; 

app.use(session({
    secret: '12345',
    resave: true,
    saveUninitialized: true,
    cookie: {
		secure: false,
		maxAge: maxAge
	}
}))

app.use(cookieParser());
app.use(express.json());
app.use(express.urlencoded({ extended: false }));
app.use(
  cors({
    origin: 'http://localhost:3000',
    credentials: true,
  })
); //cors 설정을 한다..

app.listen(PORT, () => {
  console.log(`Listening on port ${PORT}`);
}); // 서버 실행

app.post('/login', async (req, res) => {
	/*
	TODO: 토큰의 무결성 체크
	토큰이 이상이 없다면, 로그인/회원가입 로직을 수행 후 jwt 쿠키를 보낸다.
	*/
	const expires =  moment().add(sessionTime.toString(),'m').toDate();

	// 정보가 없다면 회원 가입 (강제?)
  const user = await userService.findUserByEmail(req.body.email);
	if (!user) { // 유저가 없다면 회원 가입 후 세션 생성
		let userProfilePicture = req.body.picture || null;
    await userService.createUser({
      nickname: req.body.name,
      email: req.body.email,
      google: {
        id: req.body.sub,
        profileUrl: userProfilePicture,
      },
    });
		console.log('new user saved!')
	}
	console.log('login done')

	req.session.sessionid = req.body; //프론트에서 건네받은 JWT로 세션 생성
	res.cookie('name', req.body.name, {expires}); //사용자 이름 쿠키
	

	res.send(req.body.name); // 이름 보내서 뭐하게?

});

app.get("/logout", (req, res) => {
	console.log("로그아웃");
	if (req.session.sessionid) {
		
		console.log("로그아웃중입니다!");
		req.session.destroy((err) => {
			if (err) {
				console.log("세션 삭제시에 에러가 발생했습니다.");
				return;
			}
			console.log("세션이 삭제됐습니다.");
		});
		res.clearCookie('name');
		res.send(req.body.name);
	} else {
		console.log("로그인이 안돼있으시네요?");
		res.send(req.body.name);
	}
});

app.post("/post", async (req, res) => {
	console.log("포스팅 '해줘'");
	if(!req.session.sessionid){
		// 세션이 없엉
	}

	const inputTitle = req.body.title
	const inputContent = req.body.content
	const inputImage = []
	const useremail = req.session.sessionid.email
  const author = await userService.findUserByEmail(useremail);
  console.log(author)
  await articleService.createArticle({
		title: inputTitle,
		content: inputContent,
		imageUrls: inputImage,
		author: author,
		comments: [],
		likes: []
	});
  
	console.log('saved.')
	res.send();
});

app.get("/check", (req, res) => {
	if(req.session.sessionid){
	  res.send(true);
	}
  else {
	  res.send(false);
  }
});

app.get("/loadarticle", async (req, res) => {
  if(req.session.sessionid){	
	console.log("세션 O")
  }
  else {
	console.log("세션 X")
  }
  const articles = await articleService.findAllArticle();
  res.send(JSON.stringify(articles) );
});










